﻿let ParametersInTextHelper = {

    parametersDataDictionary: {},

    /**
     * apply tagify module with parameters variables
     * @param {HTMLElement} textInputElement
     * @param {object} previousParameters
     */
    applyTagify: function (textInputElement) {
        
        let inputParametersData = this.parametersDataDictionary[textInputElement.id];

        let previousParameters = getPreviousBlockParameters();

        if (!inputParametersData) {

            let tagify = new Tagify(textInputElement, {
                mode: 'mix',
                pattern: /\$/,
                tagTextProp: 'paramDisplayName',
                duplicates: true,
                enforceWhitelist: false,
                whitelist: this.getParametersRepresentation(previousParameters),
                dropdown: {
                    enabled: 0,
                    position: 'text',
                    mapValueTo: 'paramDisplayName'
                },
                transformTag: function (tagData) {
                    if (tagData.type === "Block") {
                        let parameterData = this.whitelist.find(f => f.blockId === tagData.blockId && f.paramName === tagData.paramName);

                        if (parameterData) {
                            tagData.blockName = parameterData.blockName;
                            tagData.paramDisplayName = "Block." + parameterData.blockName + "." + parameterData.paramName;
                        }
                        else
                            this.removeTags(tagData);
                    }
                    else if (tagData.type === "Data") {
                        let parameterData = this.whitelist.find(f => f.dataId === tagData.dataId);

                        if (!parameterData)
                            this.removeTags(tagData);
                    }
                }
            })

            tagify.on('input', function (e) {
                var prefix = e.detail.prefix;

                if (prefix) {
                    let previousParameters = getPreviousBlockParameters();
                    tagify.whitelist = ParametersInTextHelper.getParametersRepresentation(previousParameters);

                    if (e.detail.value.length > 1)
                        tagify.dropdown.show(e.detail.value);
                }
            })

            tagify.on('add', function (e) {
                console.log(e)
            })

            this.parametersDataDictionary[textInputElement.id] = tagify;
        }
        else {
            inputParametersData.whitelist = this.getParametersRepresentation(previousParameters);
        }
    },

    /**
     * get a unified parameter representation
     * @param {object} previousBlockParameters
     */
    getParametersRepresentation(previousBlockParameters) {

        let parametersRepresentation = [];

        for (let blockId of Object.keys(previousBlockParameters)) {

            let blockParamData = previousBlockParameters[blockId];

            if (blockParamData) {
                for (let blockParam of blockParamData) {

                    let block = canvas.view.getFigure(blockId);

                    parametersRepresentation.push({
                        type: "Block",
                        blockId: blockId,
                        blockName: block.userData.blockName,
                        paramName: blockParam,
                        value: "Block." + blockId + "." + blockParam,
                        paramDisplayName: "Block." + block.userData.blockName + "." + blockParam
                    });
                }
            }
        }

        let dataSelect = document.getElementById("dataSelectViewBag");

        for (let option of dataSelect.children) {
            parametersRepresentation.push({
                type: "Data",
                dataId: option.value,
                paramName: option.text,
                value: "Data." + option.value,
                paramDisplayName: "Data." + option.text
            });
        }

        return parametersRepresentation;
    },

    getCKEditorParametersList: function (queryText) {
        return new Promise(resolve => {
            let previousParameters = getPreviousBlockParameters();
            let parametersRepresentation = ParametersInTextHelper.getParametersRepresentation(previousParameters);

            let itemsToDisplay = parametersRepresentation
                .filter(f => f.paramDisplayName.toLowerCase().includes(queryText.toLowerCase()))
                .map(m => { m.id = "$" + m.paramDisplayName; return m; })
                .slice(0, 10);

            resolve(itemsToDisplay);
        });
    },

    getCKEditorTagElement: function (item) {
        let itemElement = document.createElement("span");

        itemElement.textContent = item.paramDisplayName;
        itemElement.dataset.parameterJson = JSON.stringify(item);

        return itemElement;
    },

    applyCKEditorMentionCustomization: function (editor) {
        editor.conversion.for('upcast').elementToAttribute({
            view: {
                name: 'span',
                key: 'data-mention',
                classes: 'mention'
            },
            model: {
                key: 'mention',
                value: viewItem => {
                    let parametersJsonValue = viewItem.getAttribute("data-parameters-json");
                    let parametersJson = JSON.parse(parametersJsonValue);

                    const mentionAttribute = editor.plugins.get('Mention').toMentionAttribute(viewItem, {
                        type: parametersJson.type,
                        blockId: parametersJson.blockId,
                        dataId: parametersJson.dataId,
                        blockName: parametersJson.blockName,
                        paramName: parametersJson.paramName,
                        value: parametersJson.value,
                        paramDisplayName: parametersJson.paramDisplayName
                    });

                    return mentionAttribute;
                }
            },
            converterPriority: 'high'
        });

        editor.conversion.for('downcast').attributeToElement({
            model: 'mention',
            view: (modelAttributeValue, writer) => {
                if (!modelAttributeValue) {
                    return;
                }

                let spanElement = writer.createAttributeElement('span', {
                    class: 'mention',
                    'data-parameters-json': JSON.stringify(modelAttributeValue)
                }, {
                    // Make mention attribute to be wrapped by other attribute elements.
                    priority: 20,
                    // Prevent merging mentions together.
                    id: modelAttributeValue.uid
                });

                return spanElement;
            },
            converterPriority: 'high'
        });
    },

    convertCKEditorTagsToCommonFormat: function (CKEditorText) {
        let resultText = CKEditorText;

        let tagSearchPattern = new RegExp("<span.+?data-parameters-json=\\\"(\\{.*?\\}).+?<\\/span>", "g");
        let match = tagSearchPattern.exec(CKEditorText);

        while (match != null) {
            resultText = resultText.replace(match[0], "[[" + match[1] + "]]");
            match = tagSearchPattern.exec(CKEditorText);
        }

        return $('<textarea/>').html(resultText).text();
    },

    convertCommonTagsToCKEditorTags: function (taggedText) {
        let resultText = taggedText;

        let tagSearchPattern = new RegExp("\\[\\[(.*?)\\]\\]", "g");
        let match = tagSearchPattern.exec(taggedText);

        while (match != null) {
            let decodedJsonString = $('<textarea/>').html(match[1]).text();
            let parametersJson = JSON.parse(decodedJsonString);
            let tagText = "$" + parametersJson.paramDisplayName;

            resultText = resultText.replace(match[0], '<span class="mention" data-parameters-json="' + this.escapeHtml(match[1]) + '">' + tagText + '</span>');
            match = tagSearchPattern.exec(taggedText);
        }

        return resultText;
    },

    escapeHtml: function (text) {
        return text
            .replace(/&/g, "&amp;")
            .replace(/</g, "&lt;")
            .replace(/>/g, "&gt;")
            .replace(/"/g, "&quot;")
            .replace(/'/g, "&#039;");
    }
}